(function() {
  var ViewModel,
    __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  ViewModel = (function() {
    function ViewModel(io, config) {
      this.io = io;
      this.config = config;
      this.clearAlert = __bind(this.clearAlert, this);
      this.running = ko.observable(false);
      this.pattern = ko.observable("");
      this.events = ko.observableArray([]);
      this.unexpectedError = ko.observable(false);
      this.alreadySniffing = ko.observable(false);
      this.showTable = ko.computed((function(_this) {
        return function() {
          return _this.events().length > 0;
        };
      })(this));
      this.canStart = ko.computed((function(_this) {
        return function() {
          return !_this.running() && _this.pattern().length > 0;
        };
      })(this));
    }

    ViewModel.prototype.start = function() {
      this.clearAlert();
      this.events([]);
      return $.ajax('/pattern', {
        type: 'POST',
        data: {
          pattern: this.pattern(),
          rocketio_session: this.io.session
        },
        success: (function(_this) {
          return function() {
            _this.io.on('event', function(events) {
              return _this.rotate(_.map(events, function(e) {
                return _this.format(e);
              }));
            });
            return _this.running(true);
          };
        })(this),
        error: (function(_this) {
          return function(jqXHR, textStatus, errorThrown) {
            if (jqXHR.status === 503) {
              return _this.alreadySniffing(true);
            } else {
              return _this.unexpectedError(true);
            }
          };
        })(this)
      });
    };

    ViewModel.prototype.stop = function() {
      this.clearAlert();
      return $.ajax('/pattern', {
        type: 'DELETE',
        success: (function(_this) {
          return function() {
            _this.io.removeListener('event');
            return _this.running(false);
          };
        })(this),
        error: (function(_this) {
          return function(jqXHR, textStatus, errorThrown) {
            return _this.unexpectedError(true);
          };
        })(this)
      });
    };

    ViewModel.prototype.selectPattern = function(data, event) {
      this.clearAlert();
      this.pattern(event.toElement.innerText);
      if (this.running()) {
        return this.stopThenStart();
      } else {
        return this.start();
      }
    };

    ViewModel.prototype.submit = function() {
      this.clearAlert();
      if (this.pattern().length > 0) {
        if (this.running()) {
          return this.stopThenStart();
        } else {
          return this.start();
        }
      }
    };

    ViewModel.prototype.stopThenStart = function() {
      return $.ajax('/pattern', {
        type: 'DELETE',
        success: (function(_this) {
          return function() {
            _this.io.removeListener('event');
            _this.running(false);
            return _this.start();
          };
        })(this),
        error: (function(_this) {
          return function(jqXHR, textStatus, errorThrown) {
            return _this.unexpectedError(true);
          };
        })(this)
      });
    };

    ViewModel.prototype.clearAlert = function() {
      this.unexpectedError(false);
      return this.alreadySniffing(false);
    };

    ViewModel.prototype.format = function(event) {
      return {
        tag: event.tag,
        time: (new Date(event.time * 1000)).toLocaleString(),
        record: JSON.stringify(event.record)
      };
    };

    ViewModel.prototype.rotate = function(events) {
      var current, tmp;
      current = this.events();
      tmp = current.concat(events);
      if (tmp.length > this.config.max_events) {
        return this.events(tmp.slice(tmp.length - this.config.max_events, tmp.length));
      } else {
        return this.events(tmp);
      }
    };

    return ViewModel;

  })();

  $(function() {
    var config, config_elem, io;
    config_elem = $('#config');
    config = {
      max_events: config_elem.data('max-events')
    };
    io = new RocketIO().connect();
    return ko.applyBindings(new ViewModel(io, config));
  });

}).call(this);
